#include <limits>
#include <string>

using namespace std;

#include <tparamxx/tparamxx.h>

#include <tracefctxx/TraceFct.h>

#include "imageblur.h"

static string getstring( paramfile pf, char* pname ) {

    char dummy[1024+1];

    tpgetstr(pf, pname, dummy, sizeof(dummy) - 1, 1);

    return string(dummy);
}

void
gparam(int argc, char *argv[], Params &pars)
{
    TraceFct tf("gparam");

    paramfile pf = tparamopen(NULL, argv, argc, "r");

    pars.version = tpgetb(pf, "version");
    pars.help = tpgetb(pf, "help");

    if ( pars.version || pars.help)
	return;


    pars.input  = getstring( pf, "input" );
    pars.output = getstring( pf, "output" );
    pars.pdf    = getstring( pf, "pdf" );

    pars.pmin = tpgetd( pf, "pmin" );
    pars.pmax = tpgetd( pf, "pmax" );

    pars.clip = getstring(pf, "clip" );

    string offset;

    offset = getstring( pf, "x0" );
    pars.xoffset =
	offset == "center"  ? numeric_limits<double>::max()
	                  : tpgetd( pf, "x0" );


    offset = getstring( pf, "y0" );
    pars.yoffset =
	offset == "center"  ? numeric_limits<double>::max()
	                  : tpgetd( pf, "y0" );

    pars.scale = tpgetd( pf, "scale" );

    pars.seed1 = tpgeti( pf, "seed1" );
    pars.seed2 = tpgeti( pf, "seed2" );
    pars.block = tpgeti( pf, "block" );

    if (pars.seed1 < 1 || pars.seed1 > 2147483562)
	tf.die( "seed1 must have range [1,2147483562]");

    if (pars.seed2 < 1 || pars.seed2 > 2147483398)
	tf.die( "seed2 must have range [1,2147483398]");

    if (pars.block < 0 || pars.block > 1048575)
	tf.die( "block must have range [0,1048575]");

    paramclose(pf);
}
